class FILE:
    class EXTENSION:
        EXCEL_OLD: str = "xls"
        EXCEL_NEW: str = "xlsx"
        JPEG: str = "jpeg"
        JPG: str = "jpg"
        PNG: str = "png"
        PDF: str = "pdf"
        TXT: str = "txt"
        JSON: str = "json"
        PYTHON: str = "py"
        BASE64: str = "base64"
        EXE: str = "exe"
        ARCHIVE: str = "zip"
        TRUE_TYPE_FONT: str = "ttf"
        WHEEL: str = "whl"